/**
 * Binární strom.
 */
public class Tree {

	// Třída uzlu je soukromá - ostatní třídy nevidí do implementace
	// stromu.
	/**
	 * Jeden uzel stromu.
	 */
	private static class TreeNode {

		// package přístup - třída Tree přistupuje přímo
		/**
		 * Levý podstrom.
		 */
		TreeNode leftNode;

		/**
		 * Hodnota uložená v uzlu.
		 */
		int data;

		/**
		 * Pravý podstrom.
		 */
		TreeNode rightNode;

		/**
		 * Vytvoří nový list stromu.
		 * 
		 * @param nodeData
		 *            data uložená do nového uzlu.
		 */
		public TreeNode(int nodeData) {
			data = nodeData;
			leftNode = null;
			rightNode = null;
		}

		/**
		 * Vloží nový uzel s danou hodnotou do podstromu začínajícího
		 * tímto uzlem. Ignoruje duplicitní hodnoty.
		 * 
		 * @param insertValue
		 *            vkládaná hodnota.
		 */
		public void insert(int insertValue) {
			if (insertValue < data) {
				// vložení do levého podstromu
				if (leftNode == null) {
					// vytvoření nového podstromu
					leftNode = new TreeNode(insertValue);
				} else {
					// rekurzivní volání
					leftNode.insert(insertValue);
				}
			} else if (insertValue > data) {
				// vložení do pravého podstromu
				if (rightNode == null) {
					// vytvoření nového podstromu
					rightNode = new TreeNode(insertValue);
				} else {
					// rekurzivní volání
					rightNode.insert(insertValue);
				}
			} else {
				// duplicitní hodnota - nedělej nic
			}
		}

	}

	/**
	 * Kořen stromu.
	 */
	private TreeNode root;

	/**
	 * Vytvoří prázdný strom.
	 */
	public Tree() {
		root = null;
	}

	/**
	 * Vloží novou hodnotu do binárního vyhledávacího stromu.
	 * 
	 * @param insertValue
	 *            vkládaná hodnota.
	 */
	public void insertNode(int insertValue) {
		if (root == null) {
			root = new TreeNode(insertValue);
		} else {
			root.insert(insertValue);
		}
	}

	/**
	 * Vytiskne hodnoty uzlů stromu v pořadí preorder.
	 */
	public void preorderTraversal() {
		preorderHelper(root);
	}

	/**
	 * Pomocná rekurzivní metoda pro procházení uzlů stromu v pořadí
	 * preorder.
	 * 
	 * @param node
	 *            kořen podstromu, který se prochází.
	 */
	private void preorderHelper(TreeNode node) {
		if (node == null) {
			return;
		}
		// tisk hodnoty
		System.out.printf("%d ", node.data);
		// projití levého podstromu
		preorderHelper(node.leftNode);
		// projití pravého podstromu
		preorderHelper(node.rightNode);
	}

	/**
	 * Vytiskne hodnoty uzlů stromu v pořadí inorder.
	 */
	public void inorderTraversal() {
		inorderHelper(root);
	}

	/**
	 * Pomocná rekurzivní metoda pro procházení uzlů stromu v pořadí
	 * inorder.
	 * 
	 * @param node
	 *            kořen podstromu, který se prochází.
	 */
	private void inorderHelper(TreeNode node) {
		if (node == null) {
			return;
		}
		// projití levého podstromu
		inorderHelper(node.leftNode);
		// tisk hodnoty
		System.out.printf("%d ", node.data);
		// projití pravého podstromu
		inorderHelper(node.rightNode);
	}

	/**
	 * Vytiskne hodnoty uzlů stromu v pořadí postorder.
	 */
	public void postorderTraversal() {
		postorderHelper(root);
	}

	/**
	 * Pomocná rekurzivní metoda pro procházení uzlů stromu v pořadí
	 * postorder.
	 * 
	 * @param node
	 *            kořen podstromu, který se prochází.
	 */
	private void postorderHelper(TreeNode node) {
		if (node == null) {
			return;
		}
		// projití levého podstromu
		postorderHelper(node.leftNode);
		// projití pravého podstromu
		postorderHelper(node.rightNode);
		// tisk hodnoty
		System.out.printf("%d ", node.data);
	}

        void levelOrder() {
            Queue queue = new Queue();
            if(null == root) {
                return;
            }
            queue.enqueue(root);
            while(!queue.isEmpty()) {
                TreeNode node = (TreeNode) queue.dequeue();
                System.out.printf("%d ", node.data);
                if(node.leftNode != null)
                    queue.enqueue(node.leftNode);
                if(node.rightNode != null)
                    queue.enqueue(node.rightNode);
            }
        }

        void outputTree() {
            outputTreeHelper(root,0);
        }

        void outputTreeHelper(TreeNode node,int totalSpaces) {
            if(node == null) return;
            outputTreeHelper(node.rightNode,totalSpaces + 5);
            System.out.printf("%"+(totalSpaces+2)+"d\n",node.data);
            outputTreeHelper(node.leftNode,totalSpaces + 5);

        }
}